uniform sampler2D field;
uniform sampler2D noise1;
uniform sampler2D noise2;
uniform sampler2D normals;

uniform float time;

varying vec3 posWorldC;
varying vec3 posEyeC;
varying vec3 posTexC;
varying vec3 normal;

vec3 getNoise(vec2 loc, int index, int len)
{
	float depth = time + float(index + len - 1) / float(2 * len - 1);
	depth = mod(depth, 1);
	//if (depth > 1)
	//	depth = 1 - depth;
	
	vec3 color1 = vec3(texture2D(noise1, loc));
	vec3 color2 = vec3(texture2D(noise2, loc));
	
	return mix(color1, color2, depth);
}

// This shader does the actual LIC tracing and convolution

void main (void){
	//int res = 512;
	//int res = 1024;
	int res = 1024;
	//float texScale = 1.5;
	float texScale = 3;
	int len = 40;
	
	vec2 st0 = vec2(posTexC);
	vec2 st;
	vec3 color;
	
	float oneOverNum = 1./(len+len+1.);	
	vec2 v;
	vec3 K3Screen;
	float dmax = 1./res;
	
	st = st0;
	
	//vec3 texCoord1 = vec3(texScale*st, time + float(len - 1) / float(2 * len - 1));
	//color = vec3(texture3D(noise, texCoord1));
	color = getNoise(texScale*st, 0, len);

	//st = st0; // <- Used to be uninitialized
	
	if(length(2 * vec3(texture2D(field, st)) - 1) > 1e-2)
	{
	for(int i = 0; i < len; i++){
		K3Screen = vec3(texture2D(field, st));
		K3Screen *= 2.0;
		K3Screen -= 1.0;

		v = vec2(K3Screen);
		v = normalize(v);

		v *= dmax;
		st += v;
		//color += vec3(texture2D(noise,  texScale*st));
		//vec3 texCoord = vec3(texScale*st, time + float(i + len - 1) / float(2 * len - 1));
		//color += vec3(texture3D(noise, texCoord));
		color += getNoise(texScale*st, i, len);
	}

	st = st0;

	for(int i = 0; i < len; i++){
		K3Screen = vec3(texture2D(field, st));
		K3Screen *= 2.0;
		K3Screen -= 1.0;

		v = vec2(K3Screen);
		v = normalize(v);

		v *= dmax;
		st -= vec2(v);
		//color += vec3(texture2D(noise,  texScale*st));
		//vec3 texCoord = vec3(texScale*st, time + float(len - 1 - i) / float(2 * len - 1));
		//color += vec3(texture3D(noise, texCoord));
		color += getNoise(texScale*st, -i, len);
	}

	color *= oneOverNum;
	}

	gl_FragColor = vec4(color, 1.0);
}

